{****************************************************}
{                MOUSE FRIENDLY PACKAGE              }
{                ~~~~~~~~~~~~~~~~~~~~~~              }
{                     version 1.20                   }
{                                                    }
{ (c) 1998 by Sergey Kirichenko (ksv@cheerful.com)   }
{                                                    }
{  in this release: TMFLabel                         }
{                   TMFPanel                         }
{                   TMFBevel                         }
{                   TMFBitBtn                        }
{                   TMFImage                         }
{                   TMFArea                          }
{                                                    }
{****************************************************}

unit MouseFriendly;


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, DsgnIntf;

type
  TAboutMouseFriendly = class(TPropertyEditor)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
    function GetValue: string; override;
  end;

{- TMFArea -}
  TMFArea = class(TCustomLabel)
  private
    FAbout: TAboutMouseFriendly;
    FMouseFriendy: boolean;
    FOnMouseEnterEvent, FOnMouseExitEvent: TNotifyEvent;
  protected
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit (var Message: TMessage); message CM_MOUSELEAVE;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure MouseEnterEvent;
    procedure MouseExitEvent;
  published
    property About: TAboutMouseFriendly read FAbout write FAbout;
    property EnabledMouseFriendy: Boolean read FMouseFriendy write FMouseFriendy default True;
    property Align;
    property Enabled;
    property FocusControl;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnterEvent write FOnMouseEnterEvent;
    property OnMouseExit: TNotifyEvent read FOnMouseExitEvent write FOnMouseExitEvent;
  end;

{- TMFLabel -}
  TMFLabel = class(TLabel)
  private
    FAbout: TAboutMouseFriendly;
    FMouseFriendy: boolean;
    FOnMouseColor: TColor;
    FOnMouseEnterEvent, FOnMouseExitEvent: TNotifyEvent;
    FFontOnMouse: TFont;
    FOnMouseExitColor: TColor;
    FFontOnMouseExit: TFont;
    procedure SetFont(Value: TFont);
  protected
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit (var Message: TMessage); message CM_MOUSELEAVE;
  public
    procedure MouseEnterEvent;
    procedure MouseExitEvent;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property About: TAboutMouseFriendly read FAbout write FAbout;
    property EnabledMouseFriendy: Boolean read FMouseFriendy write FMouseFriendy;
    property ColorOnMouse: TColor read FOnMouseColor write FOnMouseColor;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnterEvent write FOnMouseEnterEvent;
    property OnMouseExit: TNotifyEvent read FOnMouseExitEvent write FOnMouseExitEvent;
    property FontOnMouse: TFont read FFontOnMouse write SetFont;
  end;
{- TMFPanel -}
  TMFPanel = class(TPanel)
  private
    FAbout: TAboutMouseFriendly;
    FMouseFriendy: boolean;
    FOnMouseColor, FOnMouseFontColor: TColor;
    FOnMouseEnter, FOnMouseExit: TNotifyEvent;
    FBevelInnerOnMouse,FBevelOuterOnMouse: TPanelBevel;
    FOnMouseExitColor,FOnMouseExitFontColor: TColor;
    FBevelInnerOnMouseExit,FBevelOuterOnMouseExit: TPanelBevel;
  protected
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit (var Message: TMessage); message CM_MOUSELEAVE;
  public
    procedure MouseEnter;
    procedure MouseExit;
    constructor Create(AOwner: TComponent); override;
  published
    property About: TAboutMouseFriendly read FAbout write FAbout;
    property EnabledMouseFriendy: Boolean read FMouseFriendy write FMouseFriendy;
    property ColorOnMouse: TColor read FOnMouseColor write FOnMouseColor;
    property FontColorOnMouse: TColor read FOnMouseFontColor write FOnMouseFontColor;
    property BevelInnerOnMouse: TPanelBevel read FBevelInnerOnMouse write FBevelInnerOnMouse;
    property BevelOuterOnMouse: TPanelBevel read FBevelOuterOnMouse write FBevelOuterOnMouse;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseExit: TNotifyEvent read FOnMouseExit write FOnMouseExit;
  end;

{- TMFBevel -}
  TMFBevel = class(TBevel)
  private
    FAbout: TAboutMouseFriendly;
    FStyleOnMouseExit: TBevelStyle;
    FOnMouseEnter, FOnMouseExit: TNotifyEvent;
    FStyleOnMouse: TBevelStyle;
    FMouseFriendy: boolean;
  protected
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit (var Message: TMessage); message CM_MOUSELEAVE;
  public
    procedure MouseEnter;
    procedure MouseExit;
    constructor Create(AOwner: TComponent); override;
  published
    property About: TAboutMouseFriendly read FAbout write FAbout;
    property EnabledMouseFriendy: Boolean read FMouseFriendy write FMouseFriendy;
    property StyleOnMouse: TBevelStyle read FStyleOnMouse write FStyleOnMouse;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseExit: TNotifyEvent read FOnMouseExit write FOnMouseExit;
  end;
{- TMFBitBtn -}
  TMFBitBtn = class(TBitBtn)
  private
    FAbout: TAboutMouseFriendly;
    FMouseFriendy: boolean;
    FOnMouseEnter, FOnMouseExit: TNotifyEvent;
    FFontOnMouse: TFont;
    FGlyphOnMouse: TBitmap;
    FGlyphOnMouseExit: TBitmap;
    FFontOnMouseExit: TFont;
    procedure SetFont(Value: TFont);
    procedure SetGlyph(Value: TBitmap);
  protected
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit (var Message: TMessage); message CM_MOUSELEAVE;
  public
    procedure MouseEnter;
    procedure MouseExit;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property About: TAboutMouseFriendly read FAbout write FAbout;
    property EnabledMouseFriendy: Boolean read FMouseFriendy write FMouseFriendy;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseExit: TNotifyEvent read FOnMouseExit write FOnMouseExit;
    property FontOnMouse: TFont read FFontOnMouse write SetFont;
    property GlyphOnMouse: TBitmap read FGlyphOnMouse write SetGlyph;            // stored IsCustom;
  end;
{- TMFImage -}
  TMFImage = class(TImage)
  private
    FAbout: TAboutMouseFriendly;
    FMouseFriendy: boolean;
    FOnMouseEnter, FOnMouseExit: TNotifyEvent;
    FImageOnMouse,FImageOnMouseExit: TPicture;
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit (var Message: TMessage); message CM_MOUSELEAVE;
    procedure SetImage(Value: TPicture);
  protected
  public
    procedure MouseEnter;
    procedure MouseExit;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property About: TAboutMouseFriendly read FAbout write FAbout;
    property EnabledMouseFriendy: Boolean read FMouseFriendy write FMouseFriendy;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseExit: TNotifyEvent read FOnMouseExit write FOnMouseExit;
    property PictureOnMouse: TPicture read FImageOnMouse write SetImage;
  end;
(*
{- TMFEdit -}
  TMFEdit = class(TEdit)
  private
    FAbout: TAboutMouseFriendly;
    FMouseFriendy: boolean;
    FOnMouseEnter, FOnMouseExit: TNotifyEvent;
    FFontOnMouse: TFont;
    FColorOnMouse: TColor;
    FFontOnMouseExit: TFont;
    FColorOnMouseExit: TColor;
    procedure SetFont(Value: TFont);
  protected
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit (var Message: TMessage); message CM_MOUSELEAVE;
  public
    procedure MouseEnter;
    procedure MouseExit;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property About: TAboutMouseFriendly read FAbout write FAbout;
    property EnabledMouseFriendy: Boolean read FMouseFriendy write FMouseFriendy;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseExit: TNotifyEvent read FOnMouseExit write FOnMouseExit;
    property ColorOnMouse: TColor read FColorOnMouse write FColorOnMouse;
    property FontOnMouse: TFont read FFontOnMouse write SetFont;
  end;
*)
{----------------------------------------------}

procedure Register;

implementation
{----------------------------------------------}
constructor TMFArea.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AutoSize:= False;
  Transparent:= True;
  EnabledMouseFriendy:= True;
end;
{----------------------------------------------}
procedure TMFArea.Paint;
var
  Rect: TRect;
begin
  if csDesigning in ComponentState then
    with inherited Canvas do
    begin
      Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
      Rect := ClientRect;
      Brush.Style := bsClear;
      DoDrawText(Rect, DT_EXPANDTABS or DT_WORDBREAK or DT_CENTER);
    end;
end;
{----------------------------------------------}
procedure TMFArea.MouseEnterEvent;
begin
  if FMouseFriendy then
    if Enabled and assigned(FOnMouseEnterEvent) then
      FOnMouseEnterEvent (Self);
end;
{----------------------------------------------}
procedure TMFArea.MouseExitEvent;
begin
  if FMouseFriendy then
    if Enabled and Assigned(FOnMouseExitEvent) then
      FOnMouseExitEvent (Self);
end;
{----------------------------------------------}
procedure TMFArea.CMMouseExit(var Message: TMessage);
begin
  inherited;
  MouseExitEvent;
end;
{----------------------------------------------}
procedure TMFArea.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  MouseEnterEvent;
end;
{----------------------------------------------}
constructor TMFLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFontOnMouse:= TFont.Create;
  FFontOnMouseExit:= TFont.Create;
  EnabledMouseFriendy:= TRUE;
  ColorOnMouse:= clBtnFace;
  FFontOnMouse.Color:= clYellow;
end;
{----------------------------------------------}
destructor TMFLabel.Destroy;
begin
  FFontOnMouseExit.Free;
  FFontOnMouse.Free;
  inherited Destroy;
end;
{----------------------------------------------}
procedure TMFLabel.SetFont(Value: TFont);
begin
  FFontOnMouse.Assign(Value);
end;
{----------------------------------------------}
procedure TMFLabel.MouseEnterEvent;
begin
  if FMouseFriendy then
    begin
      FOnMouseExitColor:= Color;
      FFontOnMouseExit.Assign(Font);
      Color:= FOnMouseColor;
      Font.Assign(FFontOnMouse);
      if Enabled and assigned(FOnMouseEnterEvent) then
        FOnMouseEnterEvent (Self);
    end;
end;
{----------------------------------------------}
procedure TMFLabel.MouseExitEvent;
begin
  if FMouseFriendy then
    begin
      Color:= FOnMouseExitColor;
      Font.Assign(FFontOnMouseExit);
      if Enabled and Assigned(FOnMouseExitEvent) then
        FOnMouseExitEvent (Self);
    end;
end;
{----------------------------------------------}
procedure TMFLabel.CMMouseExit(var Message: TMessage);
begin
  inherited;
  MouseExitEvent;
end;
{----------------------------------------------}
procedure TMFLabel.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  MouseEnterEvent;
end;
{----------------------------------------------}
procedure TAboutMouseFriendly.Edit;
begin
 Application.MessageBox('Mouse Friendly Package ver 1.20'+#10#13+
   'by Sergey Kirichenko (ksv@cheerful.com)','* FreeWare *',MB_ICONASTERISK);
end;
function TAboutMouseFriendly.GetValue: string;
begin
 Result := 'about Mouse Friendly Package';
end;
function TAboutMouseFriendly.GetAttributes: TPropertyAttributes;
begin
 Result := [paMultiSelect, paDialog, paReadOnly];
end;
{----------------------------------------------}
constructor TMFPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  EnabledMouseFriendy:= TRUE;
  ColorOnMouse:=   clBtnFace;
  FontColorOnMouse:= clBlue;
  BevelInnerOnMouse:= bvNone;
  BevelOuter:= bvLowered;
  BevelOuterOnMouse:= bvRaised;
end;
{----------------------------------------------}
procedure TMFPanel.MouseEnter;
begin
  if FMouseFriendy then
    begin
      FOnMouseExitColor:= Color;
      FOnMouseExitFontColor:= Font.Color;
      FBevelInnerOnMouseExit:= BevelInner;
      FBevelOuterOnMouseExit:= BevelOuter;
      Color:= FOnMouseColor;
      Font.Color:= FOnMouseFontColor;
      BevelInner:= FBevelInnerOnMouse;
      BevelOuter:= FBevelOuterOnMouse;
      if Enabled and Assigned(FOnMouseEnter) then
        FOnMouseEnter (Self);
    end;
end;
{----------------------------------------------}
procedure TMFPanel.MouseExit;
begin
  if FMouseFriendy then
    begin
      Color:= FOnMouseExitColor;
      Font.Color:= FOnMouseExitFontColor;
      BevelInner:= FBevelInnerOnMouseExit;
      BevelOuter:= FBevelOuterOnMouseExit;
      if Enabled and Assigned(FOnMouseExit) then
        FOnMouseExit (Self);
    end;
end;
{----------------------------------------------}
procedure TMFPanel.CMMouseExit (var Message: TMessage);
begin
  inherited;
  MouseExit;
end;
{----------------------------------------------}
procedure TMFPanel.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  MouseEnter;
end;
{----------------------------------------------}
constructor TMFBevel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  EnabledMouseFriendy:= TRUE;
  Style:= bsLowered;
  StyleOnMouse:= bsRaised;
end;
{----------------------------------------------}
procedure TMFBevel.MouseEnter;
begin
  if FMouseFriendy then
    begin
      FStyleOnMouseExit:= Style;
      Style:= FStyleOnMouse;
      if Enabled and Assigned(FOnMouseEnter) then
        FOnMouseEnter (Self);
    end;
end;
{----------------------------------------------}
procedure TMFBevel.MouseExit;
begin
  if FMouseFriendy then
    begin
      Style:= FStyleOnMouseExit;
      if Enabled and Assigned(FOnMouseExit) then
        FOnMouseExit (Self);
    end;
end;
{----------------------------------------------}
procedure TMFBevel.CMMouseExit (var Message: TMessage);
begin
  inherited;
  MouseExit;
end;
{----------------------------------------------}
procedure TMFBevel.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  MouseEnter;
end;
{----------------------------------------------}
constructor TMFBitBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGlyphOnMouse:= TBitmap.Create;
  FGlyphOnMouseExit:= TBitmap.Create;
  FFontOnMouse:= TFont.Create;
  FFontOnMouseExit:= TFont.Create;
  EnabledMouseFriendy:= TRUE;
  FFontOnMouse.Color:= clYellow;
end;
{----------------------------------------------}
destructor TMFBitBtn.Destroy;
begin
  FFontOnMouseExit.Free;
  FFontOnMouse.Free;
  FGlyphOnMouse.Free;
  FGlyphOnMouseExit.Free;
  inherited Destroy;
end;
{----------------------------------------------}
procedure TMFBitBtn.SetGlyph(Value: TBitmap);
begin
  FGlyphOnMouse.Assign(Value);
end;
{----------------------------------------------}
procedure TMFBitBtn.SetFont(Value: TFont);
begin
  FFontOnMouse.Assign(Value);
end;
{----------------------------------------------}
procedure TMFBitBtn.MouseEnter;
begin
  if FMouseFriendy then
    begin
      if not FGlyphOnMouse.Empty then
        begin
          FGlyphOnMouseExit.Assign(Glyph);
          Glyph.Assign(FGlyphOnMouse);
        end;
      FFontOnMouseExit.Assign(Font);
      Font.Assign(FFontOnMouse);
      if Enabled and Assigned(FOnMouseEnter) then
        FOnMouseEnter (Self);
    end;
end;
{----------------------------------------------}
procedure TMFBitBtn.MouseExit;
begin
  if FMouseFriendy then
    begin
      if not FGlyphOnMouseExit.Empty then
        Glyph.Assign(FGlyphOnMouseExit);
      Font.Assign(FFontOnMouseExit);
      if Enabled and Assigned(FOnMouseExit) then
        FOnMouseExit (Self);
    end;
end;
{----------------------------------------------}
procedure TMFBitBtn.CMMouseExit (var Message: TMessage);
begin
  inherited;
  MouseExit;
end;
{----------------------------------------------}
procedure TMFBitBtn.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  MouseEnter;
end;
{----------------------------------------------}
constructor TMFImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FImageOnMouse:= TPicture.Create;
  FImageOnMouseExit:= TPicture.Create;
  EnabledMouseFriendy:= TRUE;
end;
{----------------------------------------------}
destructor TMFImage.Destroy;
begin
  FImageOnMouse.Free;
  FImageOnMouseExit.Free;
  inherited Destroy;
end;
{----------------------------------------------}
procedure TMFImage.SetImage(Value: TPicture);
begin
  FImageOnMouse.Assign(Value);
end;
{----------------------------------------------}
procedure TMFImage.MouseEnter;
begin
  if FMouseFriendy then
    begin
      if (not Picture.Bitmap.Empty) and (not FImageOnMouse.Bitmap.Empty) then
        begin
          FImageOnMouseExit.Assign(Picture);
          Picture.Assign(FImageOnMouse);
        end;
      if Enabled and Assigned(FOnMouseEnter) then
        FOnMouseEnter (Self);
    end;
end;
{----------------------------------------------}
procedure TMFImage.MouseExit;
begin
  if FMouseFriendy then
    begin
      if not FImageOnMouseExit.Bitmap.Empty then
        Picture.Assign(FImageOnMouseExit);
      if Enabled and Assigned(FOnMouseExit) then
        FOnMouseExit (Self);
    end;
end;
{----------------------------------------------}
procedure TMFImage.CMMouseExit (var Message: TMessage);
begin
  inherited;
  MouseExit;
end;
{----------------------------------------------}
procedure TMFImage.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  MouseEnter;
end;
{----------------------------------------------}
(*
constructor TMFEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFontOnMouse:= TFont.Create;
  FFontOnMouseExit:= TFont.Create;
  EnabledMouseFriendy:= TRUE;
  FFontOnMouse.Color:= clYellow;
  ColorOnMouse:= clOlive;
end;
{----------------------------------------------}
destructor TMFEdit.Destroy;
begin
  FFontOnMouseExit.Free;
  FFontOnMouse.Free;
  inherited Destroy;
end;
{----------------------------------------------}
procedure TMFEdit.SetFont(Value: TFont);
begin
  FFontOnMouse.Assign(Value);
end;
{----------------------------------------------}
procedure TMFEdit.MouseEnter;
begin
  if FMouseFriendy then
    begin
      FColorOnMouseExit:= Color;
      Color:= FColorOnMouse;
      FFontOnMouseExit.Assign(Font);
      Font.Assign(FFontOnMouse);
      if Enabled and Assigned(FOnMouseEnter) then
        FOnMouseEnter (Self);
    end;
end;
{----------------------------------------------}
procedure TMFEdit.MouseExit;
begin
  if FMouseFriendy then
    begin
      Color:= FColorOnMouseExit;
      Font.Assign(FFontOnMouseExit);
      if Enabled and Assigned(FOnMouseExit) then
        FOnMouseExit (Self);
    end;
end;
{----------------------------------------------}
procedure TMFEdit.CMMouseExit (var Message: TMessage);
begin
  inherited;
  MouseExit;
end;
{----------------------------------------------}
procedure TMFEdit.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  MouseEnter;
end;
*)
{----------------------------------------------}
procedure Register;
begin
  RegisterComponents('Mouse Friendly', [TMFArea,TMFLabel,TMFPanel,TMFBevel,TMFBitBtn,TMFImage]);    // ,TMFEdit
  RegisterPropertyEditor(TypeInfo(TAboutMouseFriendly),nil,'About',TAboutMouseFriendly);
end;

end.
